==> two-minute-hate-0.1.tar.gz: two-minute-hate-0.1/main.py <==
#!/usr/bin/python
# $Id: main.py,v 1.203 2005/09/17 11:02:42 oohara Exp $
# usage:
# ./main.py --doc _docname_ [--font _ttf_font_file_name_] [--full-screen]
#   [--anija]
# fixed-width font highly recommended
# use --anija for maximum speed

import os
import sys
import string
import pygame
from pygame.locals import *

import hsv_to_rgb
import markov

def time_string(t):
    if t < 0:
        return "00:00:00"
    millisecond = t % 1000
    second = (t / 1000) % 60
    minute = t / (1000 * 60)
    return string.zfill(str(minute), 2) + ":" + string.zfill(str(second), 2) + ":" + string.zfill(str(millisecond / 10), 2)

fontname = None
docname = ""
fullscreen = 0
anija = 0
i = 1
while i < len(sys.argv):
    if sys.argv[i] == "--font":
        if i + 1 < len(sys.argv):
            fontname = sys.argv[i + 1]
        else:
            print "missing argument for --font"
            sys.exit(1)
        i = i + 1
    elif sys.argv[i] == "--doc":
        if i + 1 < len(sys.argv):
            docname = sys.argv[i + 1]
        else:
            print "missing argument for --doc"
            sys.exit(1)
        i = i + 1
    elif sys.argv[i] == "--full-screen":
        fullscreen = 1
    elif sys.argv[i] == "--anija":
        anija = 1
    else:
        print "unknown option: %s" % sys.argv[i]
        print "usage:"
        print "%s --doc _docname_ [--font _ttf_font_file_name_]" % sys.argv[0]
        print "  [--full-screen] [--anija]"
        sys.exit(1)
    i = i + 1

pygame.init()
flags = 0
if fullscreen != 0:
    flags = FULLSCREEN
screen = pygame.display.set_mode((480, 360), flags)
pygame.display.set_caption("two minute hate")
pygame.mouse.set_visible(0)

markov_instance = markov.markov(docname)

background = []
for j in range(8):
    hue = 210 + j * 5
    background.append(pygame.Surface(screen.get_size()))
    background[j] = background[j].convert()
    result_from = hsv_to_rgb.hsv_to_rgb((hue, 40, 85))
    result_to = hsv_to_rgb.hsv_to_rgb((hue, 25, 85))
    for i in range(360):
        red = (result_from[0][0] * (360 - i) + result_to[0][0] * i) / 360
        green = (result_from[0][1] * (360 - i) + result_to[0][1] * i) / 360
        blue = (result_from[0][2] * (360 - i) + result_to[0][2] * i) / 360
        background[j].fill((red, green, blue), (0, i, 480, 1))

font_12 = pygame.font.Font(fontname, 12)
font_24 = pygame.font.Font(fontname, 24)

game_started = 0
frame = 0
column = 0
knock = -2
knock_max = 30
knock_word = 0
knock_word_max = 60
moved = 0
move_speed = 2
char_done = 0
if anija == 0:
    char_done_cap = ((char_done / 250) + 1) * 250
    if char_done_cap > 2000:
        char_done_cap = 2000
else:
    char_done_cap = 1000
word_miss_10 = 0
text_question = markov_instance.next()
text_typed = ""
text_next = markov_instance.next()
assert text_next != None

if anija != 0:
    knock_max = 4
    knock_word_max = 14
    move_speed = 24

pygame.time.wait(1)
t_origin = pygame.time.get_ticks()
while 1:
    t_begin = pygame.time.get_ticks()
    if (game_started == 0):
        t_origin = t_begin
    flag_next = 0
    for event in pygame.event.get():
        if event.type == QUIT:
            sys.exit(0)
        elif event.type == KEYDOWN:
            if event.key == K_ESCAPE:
                sys.exit(0)
            elif (event.unicode != u"") and (ord(event.unicode) >= 32) and (ord(event.unicode) < 128):
                game_started = 1
                c = str(event.unicode)
                if (c == " "):
                    flag_next = 1
                elif (text_question != None) and (len(text_typed) < len(text_question)) and (c == text_question[len(text_typed)]):
                    text_typed = text_typed + c
                    knock = knock_max
                elif (text_question == None) and (c == text_next[0]):
                    text_typed = c
                    knock = knock_max
                    text_question = text_next
                    text_next = markov_instance.next()
                    assert text_next != None
                    column = column + 1
                    if column >= 7:
                        column = 0
                    moved = 0
                    knock_word = 0
    if anija == 0:
        char_done_cap = ((char_done / 250) + 1) * 250
        if char_done_cap > 2000:
            char_done_cap = 2000
    else:
        char_done_cap = 1000
    if (text_question != None) and ((flag_next != 0) or (text_typed == text_question) or (moved > 480)):
        if (text_typed == text_question):
            if word_miss_10 > 0:
                word_miss_10 = word_miss_10 - 1
            char_done = char_done + len(text_question)
        else:
            text_typed = ""
            word_miss_10 = word_miss_10 + 10
            if word_miss_10 >= 150:
                break
        text_question = None
        moved = 0
        knock_word = knock_word_max
    if (flag_next != 0) or ((knock_word <= 0) and (text_question == None)):
        if (flag_next != 0) and (knock_word > 0) and (text_typed != ""):
            char_done = char_done + 2
            knock = knock_max
        else:
            knock = -2
        text_question = text_next
        text_typed = ""
        text_next = markov_instance.next()
        assert text_next != None
        column = column + 1
        if column >= 7:
            column = 0
        moved = 0
        knock_word = 0
    assert char_done_cap >= 250
    assert char_done_cap <= 2000
    c = (char_done_cap / 250) - 1
    if anija != 0:
        c = 7
    screen.blit(background[c], (0, 0))
    y = 10 + column * 30
    if (text_question != None) and (text_question != ""):
        r = font_24.render(text_question, 0, (0, 0, 0))
        r.set_alpha(47)
        screen.blit(r, (480 - moved, y))
        r = font_24.render(text_question, 0, (255, 255, 255))
        screen.blit(r, (0, y))
    if (text_typed != None) and (text_typed != ""):
        hue = 180 + word_miss_10
        if (knock <= 0) and (knock_word > 0):
            result = hsv_to_rgb.hsv_to_rgb((hue, 100, 100))
            r = font_24.render(text_typed, 0, (63, 63, 63))
            r.set_alpha(255 * knock_word / knock_word_max)
            screen.blit(r, (0, y))
        else:
            if knock > 0:
                c = 50 + 50 * knock / knock_max
                result = hsv_to_rgb.hsv_to_rgb((hue, 80 + 20 * knock / knock_max, 25 + 75 * knock / knock_max))
            elif knock > -2:
                result = hsv_to_rgb.hsv_to_rgb((hue, 25, 100))
            else:
                result = hsv_to_rgb.hsv_to_rgb((hue, 100, 0))
            r = font_24.render(text_typed, 0, result[0])
            screen.blit(r, (0, y))
            if len(text_typed) >= 2:
                result = hsv_to_rgb.hsv_to_rgb((hue, 0, 0))
                r = font_24.render(text_typed[:-1], 0, result[0])
                screen.blit(r, (0, y))
    if (text_next != None) and (text_next != ""):
        r = font_24.render(text_next, 0, (255, 255, 255))
        screen.blit(r, (0, (y + 30) % 210))
    r = font_24.render("miss %0.1f" % (float(word_miss_10) / 10.0), 0, (63, 63, 63))
    screen.blit(r, (10, 300))
    r = font_24.render("%d/%d" % (char_done, char_done_cap), 0, (63, 63, 63))
    screen.blit(r, (10, 330))
    if (t_begin - t_origin > 10000) and (frame >= 500):
        s = "%d (%.1ffps)" % (frame, (float(frame) * 1000.0 / float(t_begin - t_origin)))
    else:
        s = "%d (--.-fps)" % frame
    r = font_12.render(s, 0, (63, 63, 63))
    rc = r.get_rect()
    screen.blit(r, (470 - rc.width, 315))
    if anija != 0:
        r = font_12.render("--anija", 0, (63, 63, 63))
        rc = r.get_rect()
        screen.blit(r, (470 - rc.width, 300))
    r = font_24.render(time_string(t_begin - t_origin), 0, (63, 63, 63))
    rc = r.get_rect()
    screen.blit(r, (470 - rc.width, 330))
    if game_started == 0:
        r = font_12.render("press any key to start a game, or ESC to quit", 0, (63, 63, 63))
        rc = r.get_rect()
        screen.blit(r, (240 - rc.width / 2, 270))
    pygame.display.flip()
    if game_started != 0:
        frame = frame + 1
        if (anija != 0) and (char_done >= 1000):
            break
        if char_done >= 2000:
            break
        elif (char_done >= 1750) or (anija != 0):
            knock_max = 4
            knock_word_max = 6
            move_speed = 24
        elif char_done >= 1500:
            knock_max = 6
            knock_word_max = 9
            move_speed = 24
        elif char_done >= 1250:
            knock_max = 9
            knock_word_max = 14
            move_speed = 24
        elif char_done >= 1000:
            knock_max = 14
            knock_word_max = 20
            move_speed = 24
        elif char_done >= 750:
            knock_max = 20
            knock_word_max = 30
            move_speed = 24
        elif char_done >= 500:
            knock_max = 30
            knock_word_max = 60
            move_speed = 24
        elif char_done >= 390:
            knock_max = 30
            knock_word_max = 60
            move_speed = 12
        elif char_done >= 350:
            knock_max = 30
            knock_word_max = 60
            move_speed = 8 + ((char_done - 350) / 20) * 2
        elif char_done >= 250:
            knock_max = 30
            knock_word_max = 60
            move_speed = 2 + (char_done - 250) / 20
        else:
            knock_max = 30
            knock_word_max = 60
            move_speed = 2 + char_done / 50
        if knock > -2:
            knock = knock - 1
        elif knock_word > 0:
            knock_word = knock_word - 1
        if (text_question != None) and (knock <= 0) and (knock_word <= 0):
            moved = moved + move_speed
    while pygame.time.get_ticks() < t_begin + 27:
        pygame.time.wait(1)

t_end = pygame.time.get_ticks()
while 1:
    t_begin = pygame.time.get_ticks()
    for event in pygame.event.get():
        if event.type == QUIT:
            sys.exit(0)
        elif event.type == KEYDOWN:
            if event.key == K_ESCAPE:
                sys.exit(0)
    assert char_done_cap >= 250
    assert char_done_cap <= 2000
    c = (char_done_cap / 250) - 1
    if anija != 0:
        c = 7
    screen.blit(background[c], (0, 0))
    if word_miss_10 < 150:
        r = font_24.render("miss %0.1f" % (float(word_miss_10) / 10.0), 0, (63, 63, 63))
        screen.blit(r, (10, 300))
    r = font_24.render("%d/%d" % (char_done, char_done_cap), 0, (63, 63, 63))
    screen.blit(r, (10, 330))
    if (t_begin - t_origin > 10000) and (frame >= 500):
        s = "%d (%.1ffps)" % (frame, (float(frame) * 1000.0 / float(t_end - t_origin)))
    else:
        s = "%d (--.-fps)" % frame
    r = font_12.render(s, 0, (63, 63, 63))
    rc = r.get_rect()
    screen.blit(r, (470 - rc.width, 315))
    if anija != 0:
        r = font_12.render("--anija", 0, (63, 63, 63))
        rc = r.get_rect()
        screen.blit(r, (470 - rc.width, 300))
    r = font_24.render(time_string(t_end - t_origin), 0, (63, 63, 63))
    rc = r.get_rect()
    screen.blit(r, (470 - rc.width, 330))
    r = font_12.render("game over --- press ESC to quit", 0, (63, 63, 63))
    rc = r.get_rect()
    screen.blit(r, (240 - rc.width / 2, 270))
    pygame.display.flip()
    while pygame.time.get_ticks() < t_begin + 27:
        pygame.time.wait(1)

==> two-minute-hate-0.3.tar.gz: two-minute-hate-0.3/main.py <==
#!/usr/bin/python
# $Id: main.py,v 1.239 2005/11/02 15:32:43 oohara Exp $
# usage:
# ./main.py --doc _docname_ [--font _ttf_font_file_name_] [--full-screen]
#   [--anija]
# fixed-width font highly recommended
# use --anija for maximum speed

import os
import sys
import string
import pygame
from pygame.locals import *

import clock_test
import hsv_to_rgb
import markov

def time_string(t):
    if t < 0:
        return "00:00:00"
    millisecond = t % 1000
    second = (t / 1000) % 60
    minute = t / (1000 * 60)
    return string.zfill(str(minute), 2) + ":" + string.zfill(str(second), 2) + ":" + string.zfill(str(millisecond / 10), 2)

fontname = None
docname = ""
fullscreen = 0
anija = 0
antialias = 0
i = 1
while i < len(sys.argv):
    if sys.argv[i] == "--font":
        if i + 1 < len(sys.argv):
            fontname = sys.argv[i + 1]
        else:
            print "missing argument for --font"
            sys.exit(1)
        i = i + 1
    elif sys.argv[i] == "--doc":
        if i + 1 < len(sys.argv):
            docname = sys.argv[i + 1]
        else:
            print "missing argument for --doc"
            sys.exit(1)
        i = i + 1
    elif sys.argv[i] == "--full-screen":
        fullscreen = 1
    elif sys.argv[i] == "--anija":
        anija = 1
    elif sys.argv[i] == "--antialias":
        antialias = 1
    else:
        print "unknown option: %s" % sys.argv[i]
        print "usage:"
        print "%s --doc _docname_ [--font _ttf_font_file_name_]" % sys.argv[0]
        print "  [--full-screen] [--anija] [--antialias]"
        sys.exit(1)
    i = i + 1

pygame.init()
flags = 0
if fullscreen != 0:
    flags = FULLSCREEN
screen = pygame.display.set_mode((480, 360), flags)
pygame.display.set_caption("two minute hate")
pygame.mouse.set_visible(0)
redmask, greenmask, bluemask, alphamask = screen.get_masks()

markov_instance = markov.markov(docname)

background = []
for j in range(8):
    hue = 210 + j * 7
    value = 80
    background.append(pygame.Surface(screen.get_size()))
    background[j] = background[j].convert()
    result_from = hsv_to_rgb.hsv_to_rgb((hue, 40, value))
    result_to = hsv_to_rgb.hsv_to_rgb((hue, 25, value))
    for i in range(360):
        red = (result_from[0][0] * (360 - i) + result_to[0][0] * i) / 360
        green = (result_from[0][1] * (360 - i) + result_to[0][1] * i) / 360
        blue = (result_from[0][2] * (360 - i) + result_to[0][2] * i) / 360
        background[j].fill((red, green, blue), (0, i, 480, 1))
    result_from = hsv_to_rgb.hsv_to_rgb((hue, 25, value))
    result_to = hsv_to_rgb.hsv_to_rgb((hue, 10, value))
    for i in range(j):
        y = 35 + i * 30
        red = (result_from[0][0] * (360 - y) + result_to[0][0] * y) / 360
        green = (result_from[0][1] * (360 - y) + result_to[0][1] * y) / 360
        blue = (result_from[0][2] * (360 - y) + result_to[0][2] * y) / 360
        background[j].fill((red, green, blue), (0, y, 480, 1))
    result_from = hsv_to_rgb.hsv_to_rgb((hue, 36, value))
    result_to = hsv_to_rgb.hsv_to_rgb((hue, 22, value))
    for i in range(j, 7, 1):
        y = 35 + i * 30
        red = (result_from[0][0] * (360 - y) + result_to[0][0] * y) / 360
        green = (result_from[0][1] * (360 - y) + result_to[0][1] * y) / 360
        blue = (result_from[0][2] * (360 - y) + result_to[0][2] * y) / 360
        background[j].fill((red, green, blue), (0, y, 480, 1))

font_12 = pygame.font.Font(fontname, 12)
font_24 = pygame.font.Font(fontname, 24)

clock_accurate = clock_test.clock_accurate()
game_started = 0
frame = 0
column = 0
knock = -2
knock_max = 30
knock_word = 0
knock_word_max = 60
moved = 0
move_speed = 2
char_done = 0
if anija == 0:
    char_done_cap = ((char_done / 250) + 1) * 250
    if char_done_cap > 2000:
        char_done_cap = 2000
else:
    char_done_cap = 1000
word_miss_10 = 0
text_question = markov_instance.next()
text_typed = ""
text_next = markov_instance.next()
assert text_next != None

if anija != 0:
    knock_max = 4
    knock_word_max = 4
    move_speed = 48

pygame.time.wait(1)
t_origin = pygame.time.get_ticks()
while 1:
    t_begin = pygame.time.get_ticks()
    if (game_started == 0):
        t_origin = t_begin
    flag_next = 0
    for event in pygame.event.get():
        if event.type == QUIT:
            sys.exit(0)
        elif event.type == KEYDOWN:
            if event.key == K_ESCAPE:
                sys.exit(0)
            elif (event.unicode != u"") and (ord(event.unicode) >= 32) and (ord(event.unicode) < 128):
                game_started = 1
                c = str(event.unicode)
                if (c == " "):
                    flag_next = 1
                elif (text_question != None) and (len(text_typed) < len(text_question)) and (c == text_question[len(text_typed)]):
                    text_typed = text_typed + c
                    knock = knock_max
                elif (text_question == None) and (c == text_next[0]):
                    text_typed = c
                    knock = knock_max
                    text_question = text_next
                    text_next = markov_instance.next()
                    assert text_next != None
                    column = column + 1
                    if column >= 7:
                        column = 0
                    moved = 0
                    knock_word = 0
    if anija == 0:
        char_done_cap = ((char_done / 250) + 1) * 250
        if char_done_cap > 2000:
            char_done_cap = 2000
    else:
        char_done_cap = 1000
    if (text_question != None) and ((flag_next != 0) or (text_typed == text_question) or (moved > 480)):
        if (text_typed == text_question):
            if word_miss_10 > 0:
                word_miss_10 = word_miss_10 - 1
            char_done = char_done + len(text_question)
        else:
            text_typed = ""
            word_miss_10 = word_miss_10 + 10
            if word_miss_10 >= 150:
                break
        text_question = None
        moved = 0
        knock_word = knock_word_max
    if (flag_next != 0) or ((knock_word <= 0) and (text_question == None)):
        if (flag_next != 0) and (knock_word > 0) and (text_typed != ""):
            char_done = char_done + 2
            knock = knock_max
        else:
            knock = -2
        text_question = text_next
        text_typed = ""
        text_next = markov_instance.next()
        assert text_next != None
        column = column + 1
        if column >= 7:
            column = 0
        moved = 0
        knock_word = 0
    assert char_done_cap >= 250
    assert char_done_cap <= 2000
    c = char_done / 250
    if anija != 0:
        c = char_done / 125
    if c < 0:
        c = 0
    # this check is necessary because there is no background[8]
    if c > 7:
        c = 7
    screen.blit(background[c], (0, 0))
    y = 10 + column * 30
    if (text_question != None) and (text_question != ""):
        r = font_24.render(text_question, antialias, (0, 0, 0))
        r_temp = pygame.Surface(r.get_size())
        r_temp.convert((redmask, greenmask, bluemask, 0))
        r_temp.blit(screen, (0, 0), (480 - moved, y, r_temp.get_width(), r_temp.get_height()))
        r_temp.blit(r, (0, 0))
        r_temp.set_alpha(47)
        screen.blit(r_temp, (480 - moved, y))
        r = font_24.render(text_question, antialias, (255, 255, 255))
        screen.blit(r, (0, y))
    if (text_typed != None) and (text_typed != ""):
        hue = 180 + word_miss_10
        if (knock <= 0) and (knock_word > 0):
            r = font_24.render(text_typed, antialias, (63, 63, 63))
            r_temp = pygame.Surface(r.get_size())
            r_temp.convert((redmask, greenmask, bluemask, 0))
            r_temp.blit(screen, (0, 0), (0, y, r_temp.get_width(), r_temp.get_height()))
            r_temp.blit(r, (0, 0))
            r_temp.set_alpha(255 * knock_word / knock_word_max)
            screen.blit(r_temp, (0, y))
        else:
            if knock > 0:
                #c = 50 + 50 * knock / knock_max
                result = hsv_to_rgb.hsv_to_rgb((hue, 80 + 20 * knock / knock_max, 25 + 75 * knock / knock_max))
            elif knock > -2:
                result = hsv_to_rgb.hsv_to_rgb((hue, 25, 100))
            else:
                result = hsv_to_rgb.hsv_to_rgb((hue, 100, 0))
            r = font_24.render(text_typed, antialias, result[0])
            screen.blit(r, (0, y))
            if len(text_typed) >= 2:
                result = hsv_to_rgb.hsv_to_rgb((hue, 0, 0))
                r = font_24.render(text_typed[:-1], antialias, result[0])
                screen.blit(r, (0, y))
    if (text_next != None) and (text_next != ""):
        if (text_question != None):
            r = font_24.render(text_next, antialias, (239, 239, 239))
        else:
            r = font_24.render(text_next, antialias, (255, 255, 255))
        screen.blit(r, (0, (y + 30) % 210))
    r = font_24.render("miss %d.%1d" % (word_miss_10 / 10, word_miss_10 % 10), antialias, (63, 63, 63))
    screen.blit(r, (10, 300))
    r = font_24.render("%d/%d" % (char_done, char_done_cap), antialias, (63, 63, 63))
    screen.blit(r, (10, 330))
    if (t_begin - t_origin > 10000) and (frame >= 500):
        s = "%d (%.1ffps)" % (frame, (float(frame) * 1000.0 / float(t_begin - t_origin)))
    else:
        s = "%d (--.-fps)" % frame
    r = font_12.render(s, antialias, (63, 63, 63))
    rc = r.get_rect()
    screen.blit(r, (470 - rc.width, 315))
    if anija != 0:
        r = font_12.render("--anija", antialias, (63, 63, 63))
        rc = r.get_rect()
        screen.blit(r, (470 - rc.width, 300))
    r = font_24.render(time_string(t_begin - t_origin), antialias, (63, 63, 63))
    rc = r.get_rect()
    screen.blit(r, (470 - rc.width, 330))
    if game_started == 0:
        r = font_12.render("press any key to start a game, or ESC to quit", antialias, (63, 63, 63))
        rc = r.get_rect()
        screen.blit(r, (240 - rc.width / 2, 270))
    pygame.display.flip()
    if game_started != 0:
        frame = frame + 1
        if (anija != 0) and (char_done >= 1000):
            break
        if char_done >= 2000:
            break
        if anija != 0:
            knock_max = 4
            knock_word_max = 4
            move_speed = 48
        elif char_done >= 1750:
            knock_max = 4
            knock_word_max = 6
            move_speed = 24
        elif char_done >= 1500:
            knock_max = 6
            knock_word_max = 9
            move_speed = 24
        elif char_done >= 1250:
            knock_max = 9
            knock_word_max = 14
            move_speed = 24
        elif char_done >= 1000:
            knock_max = 14
            knock_word_max = 20
            move_speed = 24
        elif char_done >= 750:
            knock_max = 20
            knock_word_max = 30
            move_speed = 24
        elif char_done >= 500:
            knock_max = 30
            knock_word_max = 60
            move_speed = 24
        elif char_done >= 390:
            knock_max = 30
            knock_word_max = 60
            move_speed = 12
        elif char_done >= 350:
            knock_max = 30
            knock_word_max = 60
            move_speed = 8 + ((char_done - 350) / 20) * 2
        elif char_done >= 250:
            knock_max = 30
            knock_word_max = 60
            move_speed = 2 + (char_done - 250) / 20
        else:
            knock_max = 30
            knock_word_max = 60
            move_speed = 2 + char_done / 50
        if knock > -2:
            knock = knock - 1
        elif knock_word > 0:
            knock_word = knock_word - 1
        if (text_question != None) and (knock <= 0) and (knock_word <= 0):
            moved = moved + move_speed
    if clock_accurate:
        while pygame.time.get_ticks() < t_begin + 30:
            pygame.time.wait(1)
    else:
        while pygame.time.get_ticks() < t_begin + 27:
            pygame.time.wait(1)

t_end = pygame.time.get_ticks()
while 1:
    t_begin = pygame.time.get_ticks()
    for event in pygame.event.get():
        if event.type == QUIT:
            sys.exit(0)
        elif event.type == KEYDOWN:
            if event.key == K_ESCAPE:
                sys.exit(0)
    assert char_done_cap >= 250
    assert char_done_cap <= 2000
    c = char_done / 250
    if anija != 0:
        c = char_done / 125
    if c < 0:
        c = 0
    # this check is necessary because there is no background[8]
    if c > 7:
        c = 7
    screen.blit(background[c], (0, 0))
    if word_miss_10 < 150:
        r = font_24.render("miss %d.%1d" % (word_miss_10 / 10, word_miss_10 % 10), antialias, (63, 63, 63))
        screen.blit(r, (10, 300))
    r = font_24.render("%d/%d" % (char_done, char_done_cap), antialias, (63, 63, 63))
    screen.blit(r, (10, 330))
    if (t_begin - t_origin > 10000) and (frame >= 500):
        s = "%d (%.1ffps)" % (frame, (float(frame) * 1000.0 / float(t_end - t_origin)))
    else:
        s = "%d (--.-fps)" % frame
    r = font_12.render(s, antialias, (63, 63, 63))
    rc = r.get_rect()
    screen.blit(r, (470 - rc.width, 315))
    if anija != 0:
        r = font_12.render("--anija", antialias, (63, 63, 63))
        rc = r.get_rect()
        screen.blit(r, (470 - rc.width, 300))
    r = font_24.render(time_string(t_end - t_origin), antialias, (63, 63, 63))
    rc = r.get_rect()
    screen.blit(r, (470 - rc.width, 330))
    r = font_12.render("game over --- press ESC to quit", antialias, (63, 63, 63))
    rc = r.get_rect()
    screen.blit(r, (240 - rc.width / 2, 270))
    pygame.display.flip()
    if clock_accurate:
        while pygame.time.get_ticks() < t_begin + 30:
            pygame.time.wait(1)
    else:
        while pygame.time.get_ticks() < t_begin + 27:
            pygame.time.wait(1)
